# miyabi256/core.py

# --- MIYABI256_CHARSとMIYABI256_INDEX_MAPの定義 ---
# 前回確定した256文字のリスト
MIYABI256_CHARS = [
    # ひらがな (インデックス 0-79)
    'あ', 'い', 'う', 'え', 'お', 'か', 'き', 'く', 'け', 'こ',
    'さ', 'し', 'す', 'せ', 'そ', 'な', 'に', 'ぬ', 'ね', 'の',
    'は', 'ひ', 'ふ', 'へ', 'ほ', 'ま', 'み', 'む', 'め', 'も',
    'や', 'ゆ', 'よ', 'っ', 'ざ', 'じ', 'ず', 'ぜ', 'ぞ', 'だ',
    'ぢ', 'づ', 'で', 'ど', 'ば', 'び', 'ぶ', 'べ', 'ぼ', 'ぱ',
    'ぴ', 'ぷ', 'ぺ', 'ぽ', 'ゃ', 'ゅ', 'ょ', 'が', 'ぎ', 'ぐ',
    'げ', 'ご', 'ら', 'り', 'る', 'れ', 'ろ', 'わ', 'ゐ', 'を',
    # カタカナ (インデックス 80-161)
    'ア', 'イ', 'ウ', 'エ', 'オ', 'カ', 'キ', 'ク', 'ケ', 'コ',
    'サ', 'シ', 'ス', 'セ', 'ソ', 'ナ', 'ニ', 'ヌ', 'ネ', 'ノ',
    'ハ', 'ヒ', 'フ', 'ヘ', 'ホ', 'マ', 'ミ', 'ム', 'メ', 'モ',
    'ヤ', 'ユ', 'ヨ', 'ッ', 'ザ', 'ジ', 'ズ', 'ゼ', 'ゾ', 'ダ',
    'ヂ', 'ヅ', 'デ', 'ド', 'バ', 'ビ', 'ブ', 'ベ', 'ボ', 'パ',
    'ピ', 'プ', 'ペ', 'ポ', 'ャ', 'ュ', 'ョ', 'ガ', 'ギ', 'グ',
    'ゲ', 'ゴ', 'ラ', 'リ', 'ル', 'レ', 'ロ', 'ワ', 'ヰ', 'ヲ',
    'ン', 'ー',
    # 漢字 (インデックス 162-254)
    '右', '円', '王', '下', '火', '九', '休', '玉', '月', '犬',
    '五', '口', '左', '山', '子', '四', '糸', '字', '耳', '七',
    '手', '十', '出', '小', '上', '人', '水', '正', '生', '青',
    '夕', '石', '赤', '千', '川', '先', '早', '草', '足', '村',
    '大', '男', '竹', '中', '虫', '町', '天', '田', '土', '日',
    '入', '年', '白', '八', '百', '文', '木', '本', '名', '目',
    '立', '力', '林', '六', '引', '羽', '何', '科', '夏', '家', 
    '外', '角', '丸', '汽', '弓', '牛', '魚', '京', '強', '教',
    '近', '兄', '形', '計', '元', '言', '原', '戸', '古', '午',
    '後', '工', '公', '広', '交', '光', '考', '行', '合', '谷',
    '国', '今', '会', '海', '絵',
    # パディング文字 (インデックス 255)
    '。'
]

# 文字からインデックスへの高速参照用辞書
MIYABI256_INDEX_MAP = {char: i for i, char in enumerate(MIYABI256_CHARS)}

def encode_bytes(data: bytes) -> str:
    """
    バイトデータを「みやび二五六」文字列にエンコードします。
    各バイト値をMIYABI256_CHARSのインデックスとして使用します。
    """
    encoded_chars = []
    for byte_val in data:
        encoded_chars.append(MIYABI256_CHARS[byte_val])
    return "".join(encoded_chars)

def decode_string(encoded_string: str) -> bytes:
    """
    「みやび二五六」文字列をバイトデータにデコードします。
    各文字をMIYABI256_INDEX_MAPで検索し、対応するバイト値を取得します。
    """
    decoded_bytes = bytearray()
    for char in encoded_string:
        if char not in MIYABI256_INDEX_MAP:
            raise ValueError(f"無効な「みやび二五六」文字が検出されました: '{char}'")
        decoded_bytes.append(MIYABI256_INDEX_MAP[char])
    return bytes(decoded_bytes)